﻿using System;
using System.Windows.Forms;
using Eneter.Messaging.EndPoints.Rpc;
using Eneter.Messaging.MessagingSystems.MessagingSystemBase;
using Eneter.Messaging.MessagingSystems.TcpMessagingSystem;
using Eneter.Messaging.Threading.Dispatching;
using Eneter.Messaging.Diagnostic;
using System.IO;
using System.Diagnostics;
using Eneter.Messaging.Nodes.ChannelWrapper;
using Eneter.Messaging.MessagingSystems.SynchronousMessagingSystem;

namespace CalculatorClient2Rpc
{
    public partial class FormRpc1 : Form
    {
        public interface ICalculator
        {
            double Sum(double a, double b);
            double Subtract(double a, double b);
            double Multiplay(double a, double b);
            double Divide(double a, double b);
        }

        private IRpcClient<ICalculator> myRpcClient;
		private IDuplexChannelWrapper myDuplexChannelWrapper;

        public FormRpc1()
        {
            InitializeComponent();

			// Internal messaging between Rpc client and channel wrapper.
			IMessagingSystemFactory anInternalMessaging = new SynchronousMessagingSystemFactory();

			// The service receives messages via one channel (i.e. it listens on one address).
			// The incoming messages are unwrapped on the server side.
			// Therefore the client must use wrapper to send messages via one channel.
			IChannelWrapperFactory aChannelWrapperFactory = new ChannelWrapperFactory();
			myDuplexChannelWrapper = aChannelWrapperFactory.CreateDuplexChannelWrapper();

			// Attach the input channel to the channel wrapper.
			// This input channel will be used by Rpc client.
			// Note: it must have the same channel name as on the service side.
			myDuplexChannelWrapper.AttachDuplexInputChannel(anInternalMessaging.CreateDuplexInputChannel("Rpc"));

            // Get RPC client.
            IRpcFactory anRpcFactory = new RpcFactory();
            myRpcClient = anRpcFactory.CreateClient<ICalculator>();

			// Connect the Rpc client with channel wrapper via the output channel.
			// Note: it must have the same channel name as on the service side.
			myRpcClient.AttachDuplexOutputChannel(anInternalMessaging.CreateDuplexOutputChannel("Rpc"));

            // Attach output channel to the channel wrapper and be able to communicate.
            TcpMessagingSystemFactory aMessaging = new TcpMessagingSystemFactory();
			IDuplexOutputChannel anOutputChannel = aMessaging.CreateDuplexOutputChannel("tcp://127.0.0.1:8091/");
			myDuplexChannelWrapper.AttachDuplexOutputChannel(anOutputChannel);
        }


        private void Form1_FormClosed(object sender, FormClosedEventArgs e)
        {
            // Stop listening by detaching the input channel.
            myRpcClient.DetachDuplexOutputChannel();
        }

        private void CalculateButton_Click(object sender, EventArgs e)
        {
            double a = double.Parse(Number1TextBox.Text);
            double b = double.Parse(Number2TextBox.Text);
			ResultLabel.Text = myRpcClient.Proxy.Sum(a, b).ToString();
        }
         
        private void SubtractButton_Click(object sender, EventArgs e)
        {
            double a = double.Parse(Number1TextBox.Text);
            double b = double.Parse(Number2TextBox.Text);
            ResultLabel.Text = myRpcClient.Proxy.Subtract(a, b).ToString();
        }

        private void MultiplyButton_Click(object sender, EventArgs e)
        {
            double a = double.Parse(Number1TextBox.Text);
            double b = double.Parse(Number2TextBox.Text);
            ResultLabel.Text = myRpcClient.Proxy.Multiplay(a, b).ToString();
        }

        private void DivideButton_Click(object sender, EventArgs e)
        {
            double a = double.Parse(Number1TextBox.Text);
            double b = double.Parse(Number2TextBox.Text);
            ResultLabel.Text = myRpcClient.Proxy.Divide(a, b).ToString();
        }
    }
}
